

----------------------------------------------------------------------------------------------------------------------------------
-- QuickTalents
----------------------------------------------------------------------------------------------------------------------------------
CreateFrame("Frame","QuickTalents",UIParent):RegisterEvent("ADDON_LOADED");
QuickTalents:SetScript("OnEvent",function(self)
	
	-- Validate settings
	QT_Saved = QT_Saved or {};
	for k,v in pairs({ -- default settings
		ButtonSize = {"number",28},
		ShowReagents = {"boolean",true},
		Position = {"table",{"TOPRIGHT",-5,-210}},
		BackgroundAlpha = {"number",0.5},
	}) do
		if not QT_Saved[k] or type(QT_Saved[k]) ~= v[1] then
			QT_Saved[k] = v[2];
		end
	end
	
	-- Queues a talent learn
	self.PendingLearn = nil;
	function self:Learn(t)
		PlayerTalentFrame_ClearTalentSelections();
		-- queue learn if we have to wait for unlearn
		for i = (t-((t-1)%3)),(t-((t-1)%3))+2 do
			if select(5,GetTalentInfo(i)) then
				self.PendingLearn = t;
				return;
			end
		end
		LearnTalents(t);
	end
	
	-- Setup main frame
	self:SetWidth(((QT_Saved.ButtonSize+3)*3)+1);
	self:SetHeight(((QT_Saved.ButtonSize+3)*6)+16);
	self:SetPoint(unpack(QT_Saved.Position));
	self:CreateTexture("QuickTalentsBackground"):SetAllPoints();
	QuickTalentsBackground:SetTexture( 0,0,0, QT_Saved.BackgroundAlpha );
	
	-- make moveable
	self:EnableMouse(true);
	self:SetMovable(true);
	self:RegisterForDrag("LeftButton")
	self:SetScript("OnDragStart", self.StartMoving)
	self:SetScript("OnDragStop", function()
		self:StopMovingOrSizing();
		QT_Saved.Position = {self:GetPoint()};
	end)
	
	-- Create buttons
	for i = 1,18 do
		local col,row = (i-1)%3,floor((i-1)/3);
		local f = CreateFrame("BUTTON","QuickTalentsButton"..i,self,"SecureActionButtonTemplate");
		f:SetAttribute("type","macro");
		f:SetAttribute("macrotext",
			"/click QuickTalentsOpener\n".. -- ensures the talent frame is ready for interaction
			"/click PlayerTalentFrameTalentsTalentRow"..(row+1).."Talent"..(col+1).."\n".. -- only way (?) to get the unlearn popup without taint
			"/click StaticPopup1Button1\n".. -- confirm unlearn
			"/run QuickTalents:Learn("..i..")" -- learn new talents (after pause if needed)
		);
		f:SetSize(QT_Saved.ButtonSize,QT_Saved.ButtonSize);
		f:SetPoint("TOPLEFT",2+col*(QT_Saved.ButtonSize+3),-(2+row*(QT_Saved.ButtonSize+3)));
		f:RegisterForDrag("LeftButton");
		f:SetScript("OnLeave",function() GameTooltip:Hide(); end);
		local t = f:CreateTexture("QuickTalentsButton"..i.."Texture"):SetAllPoints();
	end
	
	-- Handles the safe loading & opening of the Blizzard Talent UI
	LoadAddOn("Blizzard_TalentUI");
	CreateFrame("BUTTON","QuickTalentsOpener",self,"SecureActionButtonTemplate");
	QuickTalentsOpener:SetAttribute("type","macro");
	function QuickTalentsOpener:Update()
		self:SetAttribute("macrotext",
			"/click TalentMicroButton\n"..
			"/click PlayerTalentFrameTab2\n"..
			"/click PlayerSpecTab"..GetActiveSpecGroup().."\n"..
			"/click TalentMicroButton"
		);
	end
	QuickTalentsOpener:Update();
	QuickTalentsOpener:SetScript("OnEvent",QuickTalentsOpener.Update);
	QuickTalentsOpener:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	
	-- Pairs buttons to talents, sets button textures (also processes pending learn).
	self:SetScript("OnEvent",function(self,e)
		for i = 1,18 do
			local t = _G["QuickTalentsButton"..i.."Texture"];
			if e == "PLAYER_TALENT_UPDATE" then
				t:SetVertexColor(1,1,1,select(5,GetTalentInfo(i)) and 1 or 0.25);
				if self.PendingLearn then
					LearnTalents(self.PendingLearn);
					self.PendingLearn = nil;
				end
			elseif e == "PLAYER_ENTERING_WORLD" then
				-- apply button textures and properties
				t:SetTexture((select(2,GetTalentInfo(i))));
				t:SetTexCoord(0.075,0.925,0.075,0.925);
				
				-- setup talent tooltip & dragging
				local f = t:GetParent();
				f:SetScript("OnEnter", function(self)
					GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
					GameTooltip:SetTalent(i);
					GameTooltip:Show();
				end)
				f:SetScript("OnDragStart",function()
					PickupTalent(i);
				end);
			else
				-- grayscale icons while in combat
				SetDesaturation(t,e == "PLAYER_REGEN_DISABLED");
			end
		end
		-- update reagent count
		QuickTalentsReagents:SetText(select(2,GetTalentClearInfo()));
	end);
	self:RegisterEvent("PLAYER_TALENT_UPDATE");
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	-- Updates cooldowns
	local UpdateTimer = 0;
	self:SetScript("OnUpdate",function(self,e)
		UpdateTimer = UpdateTimer + e
		if UpdateTimer < 1 then return end
		UpdateTimer = 0;
		for i = 1,18 do
			local start, dura = GetSpellCooldown((GetTalentInfo(i)));
			if start and dura > 2 then
				local f = _G["QuickTalentsButton"..i.."Cooldown"];
				if not f then
					f = CreateFrame("Cooldown","QuickTalentsButton"..i.."Cooldown",_G["QuickTalentsButton"..i]);
					f:SetAllPoints();
					f:SetFrameStrata("MEDIUM");
					f:Show()
				end
				f:SetCooldown(start,dura);
			end
		end
	end);
	
	-- Reagent/clear count
	self:CreateFontString("QuickTalentsReagents"):SetFont("Fonts\\ARIALN.TTF", 13, "OUTLINE");
	QuickTalentsReagents:SetPoint("BOTTOMLEFT",2,2.5);
	if not QT_Saved.ShowReagents then
		QuickTalentsReagents:Hide();
	end
	
end);
